
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Text;
using System.Xml;
using System.Data;
using Microsoft.Office.Interop.Excel;
using System.Reflection;
using System.IO;
using System.Xml.XPath;
using System.Xml.Xsl;
using System.Diagnostics;


namespace WinChartinWinGrid
{
  public class ExcelFileReader
    {
        /// <summary>
        /// Methods To read Excel File and Convert it into XML and into DataTable
        /// </summary>
      public ExcelFileReader()
      {
      }
      public static System.Data.DataTable ReadExcelFile(string ExcelFile)
      {
          // Looping through Process Fields and removing the EXCEL.EXE which dosent get Released in the finally Block
          int priorSum = 0;
          int newSum = 0;
          int xlProd = 0;
          string filepath = System.Windows.Forms.Application.CommonAppDataPath + @"\TestExcel.xsl";
          try
          {
              Microsoft.Office.Interop.Excel.ApplicationClass xlApp = null;
              Workbook wb = null;
              try
              {// We loop through to make sure only the EXCEL Process created by this application is released
                  foreach (Process proc in Process.GetProcessesByName("EXCEL"))
                  {
                      priorSum += proc.Id;
                  }
                 xlApp = new Microsoft.Office.Interop.Excel.ApplicationClass();
                 foreach (Process proc in Process.GetProcessesByName("EXCEL"))
                 {
                     newSum += proc.Id;
                 }
                 xlProd = newSum - priorSum;
                 xlApp.DisplayAlerts = false;/* This is the ProcessID of the Excel Application Created by us
                                          * Note(We can use the WindowsHandle But is is above 11.0 Object Library and 
                                          * will fail if a 9.0 Excel Object LIbrary Was Used)*/

                  wb = xlApp.Workbooks.Open(ExcelFile, Missing.Value, Missing.Value, Missing.Value, Missing.Value, Missing.Value, Missing.Value,
                  Missing.Value, Missing.Value, Missing.Value, Missing.Value, Missing.Value, Missing.Value, Missing.Value, Missing.Value);

                  wb.SaveAs(filepath, XlFileFormat.xlXMLSpreadsheet, Missing.Value, Missing.Value, Type.Missing, Type.Missing, XlSaveAsAccessMode.xlNoChange,
                            XlSaveConflictResolution.xlLocalSessionChanges, true, Type.Missing,
                            Type.Missing, Type.Missing);
                  do
                  {
                      System.Windows.Forms.Application.DoEvents();
                  } while (!wb.Saved);
              }
              catch (Exception ex)
              {
                  string msg = "The ExcelFile Couldnt be opened " + ex.Message;
                  
              }
              finally
              {
                  wb.Close(true, filepath, false);
                  xlApp.ScreenUpdating = true;
                  xlApp.UserControl = false;
                  xlApp.DisplayAlerts = false;
                  if (wb != null)
                  {
                      System.Runtime.InteropServices.Marshal.ReleaseComObject(wb);
                      wb = null;
                  }
                  xlApp.Workbooks.Close();
                  System.Runtime.InteropServices.Marshal.ReleaseComObject(xlApp.Workbooks);
                  xlApp.Quit();
                  System.Runtime.InteropServices.Marshal.ReleaseComObject(xlApp);
                  xlApp = null;
                  Process proc = Process.GetProcessById(xlProd);
                  if (proc.ProcessName.Equals("EXCEL"))
                  {
                      proc.Kill();
                  }
                 GC.Collect();
              }
              System.Data.DataTable dt = new System.Data.DataTable();
              XmlDocument xc = new XmlDocument();
              xc.Load(filepath);
              XmlNamespaceManager nsmgr = new XmlNamespaceManager(xc.NameTable);
              nsmgr.AddNamespace("o", "urn:schemas-microsoft-com:office:office");
              nsmgr.AddNamespace("x", "urn:schemas-microsoft-com:office:excel");
              nsmgr.AddNamespace("ss", "urn:schemas-microsoft-com:office:spreadsheet");

              XmlElement xe = (XmlElement)xc.DocumentElement.SelectSingleNode("//ss:Worksheet/ss:Table", nsmgr);
              if (xe == null)
                  return null;
              XmlNodeList xl = xe.SelectNodes("ss:Row", nsmgr);
              int Row = -1, Col = 0;
              Dictionary<int, string> cols = new Dictionary<int, string>();
              foreach (XmlElement xi in xl)
              {
                  XmlNodeList xcells = xi.SelectNodes("ss:Cell", nsmgr);
                  Col = 0;
                  foreach (XmlElement xcell in xcells)
                  {
                      if (xcell.InnerXml.IndexOf("String") < 0)
                      {
                          try
                          {
                              xcell.InnerText = Convert.ToString(Math.Round(Decimal.Parse(xcell.InnerText, System.Globalization.NumberStyles.Float),3));
                          }
                          catch (Exception e) { }
                      }
                      if (Row == -1)
                      {
                          dt.Columns.Add(xcell.InnerText);
                          cols[Col++] = xcell.InnerText;
                      }
                      else
                      {
                          if (xcell.Attributes["ss:Index"] != null)
                          {
                              int idx = int.Parse(xcell.Attributes["ss:Index"].InnerText);
                              Col = idx - 1;
                          }

                          SetCol(dt, Row, (string)cols[Col++], xcell.InnerText, typeof(string));
                      }
                  }
                  Row++;
              }
              return dt;
          }
          catch (Exception ex)
          {
              string msg = "Please Format the Excel File and Load Again!Import aborted!!" ;
              
              return null;
          }
          finally
          {
              System.IO.File.Delete(filepath);
          }

      }

      /// <summary>
      /// Adds the Row to the Datatable Handles System.DBNull
      /// </summary>
      /// <param name="dt"></param>
      /// <param name="AcceptChanges"></param>
      /// <returns></returns>
      public static int AddRow(System.Data.DataTable dt, bool AcceptChanges)
      {
          object[] Values = new object[dt.Columns.Count];
          for (int Column = 0; Column < dt.Columns.Count; Column++)
          {
              if (!dt.Columns[Column].AllowDBNull)
              {
                  if (dt.Columns[Column].DefaultValue != null &&
                      dt.Columns[Column].DefaultValue != System.DBNull.Value)
                  {
                      Values[Column] = dt.Columns[Column].DefaultValue;
                  }
              }
          }
          dt.Rows.Add(Values);
          if (AcceptChanges)
          {
              dt.AcceptChanges();
          }
          return dt.Rows.Count - 1;
      }

      /// <summary>
      /// Sets data into datatable in safe manner of row index
      /// </summary>
      /// <param name="dt">DataTable to set</param>
      /// <param name="Row">Ordinal row index</param>
      /// <param name="ColumnName">name of column to set</param>
      /// <param name="Value">non/typed value to set</param>
      /// <param name="TypeOfValue">Becase Value can be null we must know datatype to manage default values</param>
      /// <returns></returns>
      public static DataColumn SetCol(System.Data.DataTable dt, int Row, string ColumnName,
                                      object Value, System.Type TypeOfValue)
      {
          if (dt == null || ColumnName == null || ColumnName == "")
              return null;

          if (Value == null)
              Value = System.DBNull.Value;

          int nIndex = -1;
          DataColumn dcol = null;
          bool Added = false;
          if (dt.Columns.Contains(ColumnName))
          {
              dcol = dt.Columns[ColumnName];
          }
          else
          {
              dcol = dt.Columns.Add(ColumnName, TypeOfValue);

          }
          if (dcol.ReadOnly)
              dcol.ReadOnly = false;

          nIndex = dcol.Ordinal;
          //new empty row appended
          if (dt.Rows.Count == Row && Row >= 0)
          {
              AddRow(dt, false);
              Added = true;
          }
          //one row
          if (Row >= 0)
          {
              dt.Rows[Row][nIndex] = Value;
          }
          else if (Row == -1)
          { //all rows
              try
              {
                  for (Row = 0; Row < dt.Rows.Count; Row++)
                  {
                      if (dt.Rows[Row].RowState == DataRowState.Deleted)
                      {
                          continue;
                      }
                      dt.Rows[Row][nIndex] = Value;
                  }
              }
              catch (Exception)
              {
              }

          }

          return dcol;
      }
      
    }
}
